/* 
* Rootfly:  A program for analyzing and measuring roots. 
* Copyright (c) 2005 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

// RootflyData.cpp: implementation of the RootflyData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RootflyData.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//empty constructor
RootflyData::RootflyData(){ }

//empty destructor
RootflyData::~RootflyData(){ }

// -------------------------------------------------------------------

bool RootflyData::Load(CString& filename)
{
  FILE *file;
  char temp_buf[1000];
  CString curr_version, image_format, image_directory;
  int num_tubes, num_transects, num_windows, num_sessions, num_roots, num_points;
  int checker = 0;
  int first_session;
  std::vector<CPoint> new_offsets;
  CPoint offset, point, new_diameter_center;
  char live_or_dead;
  bool new_still_alive;
  float new_diameter;
  char color_char;
  Color new_color;

//open the file given
  file = fopen(filename, "r");

//check to make sure that a file was opened
  if(file != NULL)
  {
//get and load the version
    checker = fscanf(file, "%s", temp_buf);
    curr_version = temp_buf;
    if( checker != 1 )
    {
      fclose(file);
      return 0;
    }

    m_version = curr_version.LockBuffer();

//check for correct file type
    if( m_version != "Rootfly_11" )
    {
//not the correct file format
      fclose(file);
      return 0;
    }

//get and load the image format
    checker = fscanf(file, "%s", temp_buf);
    image_format = temp_buf;
    if( checker != 1 )
    {
      fclose(file);
      return 0;
    }

    m_image_format = image_format.LockBuffer();

/* Get Image File Directory */
//	  checker = fscanf(file, "%s", temp_buf);
//	  image_directory = temp_buf;
//    image_directory = fgets(file, 200, file);
//	  if( checker != 1)

/* JLF - 09/24/06
 *   Above code is old, below is the new code that fixes
 * the bug that will disallow images to be in a directory 
 * containing spaces.
 */

    char *status;
    fgets ( temp_buf, 5, file);
    status = fgets ( temp_buf, 1000, file);
    temp_buf[strlen(temp_buf)-1] = '\0';
    image_directory = temp_buf;


    if(status == NULL)
    {
      fclose(file);
      return 0;
    }

    m_image_directory = image_directory.LockBuffer();

//get pixels per millimeter
    checker = fscanf(file, "%f", &m_mm_conversion);
    if( checker != 1 )
    {
      fclose(file);
      return 0;
    }

    checker = fscanf(file, "%d %d %d %d", &num_tubes, &num_transects, &num_windows, &num_sessions);
    if( checker != 4 )
    {
      fclose(file);
      return 0;
    }
//clear the data structure
    m_root.Clear();
    m_root.SetDimensions(num_tubes,num_transects,num_windows,num_sessions);

// repeat for each tube
    for(int curr_tube = 0; curr_tube < num_tubes; curr_tube++)
    {
// loop through each transect
      for(int curr_transect = 0; curr_transect < num_transects; curr_transect++)
      {
        checker = fscanf(file, "%*s %*2d%*c%*c"); // read the line: 'tube 01A:'
        if( checker != 0 )
        {
          fclose(file);
          return 0;
        }

// loop through each window
        for(int curr_window = 0; curr_window < num_windows; curr_window++)
        {
          RootList *new_rootlist;
          new_rootlist = m_root.GetRootList(curr_tube, curr_transect, curr_window);

          checker = fscanf(file, "%d", &num_sessions);
          if( checker != 1 )
          {
            fclose(file);
            return 0;
          }
          new_offsets.resize(num_sessions);

          new_rootlist->offset_list.resize(num_sessions);

          //loop through each session
          for(int curr_session = 0; curr_session < num_sessions; curr_session++)
          {
            checker = fscanf(file, "%d %d", &offset.x, &offset.y);
            if( checker != 2 )
            {
              fclose(file);
              return 0;
            }
            new_rootlist->offset_list.at(curr_session) = offset;
          }

// get number of roots in window
          checker = fscanf(file, "%d", &num_roots);
          if( checker != 1 )
          {
            fclose(file);
            return 0;
          }

// loop through each root, which is a RootOverTime object
          for(int curr_root = 0; curr_root < num_roots; curr_root++)
          {
            RootOverTime new_rootovertime;
            RootInstance *new_rootinstance;

            checker = fscanf(file, "%d %d %c%*s", &first_session, &num_sessions, &live_or_dead);
            if( checker != 3 )
            {
              fclose(file);
              return 0;
            }

            if(live_or_dead == 'l') // if first character is 'l' then it's living
              new_still_alive = true;
            else
              new_still_alive = false;

            if(num_sessions == 0)
            {
              new_rootinstance = new_rootovertime.CreateRootInstance(first_session - 1);
              new_rootinstance->points.clear();
              new_rootinstance->diameter = 0.0;
              new_rootinstance->diameter_center = 0.0;
              new_rootinstance->color = COLOR_BLACK;
            }

// loop through each instance for the current root
            for(curr_session = 0; curr_session < num_sessions; curr_session++)
            {
              new_rootinstance = new_rootovertime.CreateRootInstance(first_session + curr_session - 1);

              new_rootinstance->points.clear();

              checker = fscanf(file, "%d", &num_points);
              if( checker != 1 )
              {
                fclose(file);
                return 0;
              }

// loop through each point for the current root instance
              for(int curr_point = 0; curr_point < num_points; curr_point++)
              {
                checker = fscanf(file, "%d %d", &point.x, &point.y);
                if( checker != 2 )
                {
                  fclose(file);
                  return 0;
                }
                new_rootinstance->points.push_back(point);
              } // end for; loop through points

              checker = fscanf(file, "%d %d %f %c%*s", &new_diameter_center.x, &new_diameter_center.y, &new_diameter, &color_char);
              if( checker != 4 )
              {
                fclose(file);
                return 0;
              }
              new_rootinstance->diameter_center = new_diameter_center;
              new_rootinstance->diameter = (double)new_diameter;

              switch(color_char)
              {
                case 'b':
                  new_color = COLOR_BLACK;
                  break;
                case 'd':
                  new_color = COLOR_DARKBROWN;
                  break;
                case 'l':
                  new_color = COLOR_LIGHTBROWN;
                  break;
                case 'w':
                  new_color = COLOR_WHITE;
                  break;
              };       
              new_rootinstance->color = new_color;
            } // end for; root instances

            if(!new_still_alive) // set the last instance to dead if needed
            {
              if(num_sessions == 0)
                new_rootovertime.DeclareDead(first_session-1, true, true);
              else
                new_rootovertime.DeclareDead(first_session + num_sessions - 2);
            } 

            new_rootlist->push_back(new_rootovertime);
          } // end for; root over time
        } // end for; windows
      } //end for transects
    } // end for tubes
    fclose(file);
    return 1;
  } // end if; file exists
  return 0;		
} // Load()

// -------------------------------------------------------------------

void RootflyData::Save(CString& filename)
{
   FILE *file; 
	int ntubes,ntransects,nwindows,nsessions;
	int t,w,p,tr,g, rb, s;
	CString temp_str;
	RootList *current_rootlist;
	int num_points, num_rootblocks;
	RootInstance *current_rootinstance;
	CPoint offset;
	Color root_color;
	bool status;

/*		
Rootfly_11   // Note that version 11 uses Bartz format for naming image files:
Bartz_format // [experimentname]_T[%3tube]_L[%3window]_[%4year].[%2month].[%2day]_[%6timestamp]_[%3duplicate]_[%3initials].jpg
<Directory>  // Location of Pictures
1 1 2 3................................// <ntubes> <ntransects> <nwindows> <session>
tube 01A:	...........................// tube <tube number> <transect letter>
first window
2 0 0 8 7	..........................// global offset: <num-sessions> gx0 gy0 gx1 gy1
2 ....................................// number of root-blocks == number of roots for a window (num of root over time)
1 1 live .............................// first root-block: <first-frame> <number-of-frames> <status: live/dead>
3 0 0 5 7 99 88 15 16 4.56 darkbrown .// root line: <number of points> <point-list> <diameter(xc,yc)> <color>
1 1 dead .............................// second root-block:<first-frame> <number-of-frames> <status: live/dead>
2 2 1 22 31 16 17 3.12 white .........// root line: <number of points> <point-list> <diameter(xc,yc)> <color>
second window 
2 0 0 9 6 ............................// global offset: <num-sessions> gx0 gy0 gx1 gy1
0 ....................................// number of root-blocks
*/

   if( (file=fopen(filename,"w")) != NULL ) 
   {	
      // write version
		fprintf(file,"%s\n",m_version);
		// write image format
		fprintf(file,"%s\n",m_image_format);

/* 05/19/06 - Added Picture Directory Location - JLF */
		fprintf(file,"%s\n",m_image_directory);

    //pixels per millimeter
    fprintf(file,"%.2f\n",m_mm_conversion);
		// write number of tubes transect windows sessions
		m_root.GetDimensions(&ntubes,&ntransects,&nwindows,&nsessions);
		fprintf(file,"%d %d %d %d\n",ntubes,ntransects,nwindows,nsessions);
		
		// loop through tubes
		for( t = 0; t < ntubes; t++ ) 
      {
			// loop through transects
			for( tr = 0; tr < ntransects; tr++ ) 
         {
				// write tube 01A:
  			temp_str.Format("tube %02d%c:",t+1,65+tr);
				fprintf(file,"%s\n",temp_str);

				// loop through windows
				for( w = 0; w < nwindows; w++ ) 
            {
					current_rootlist = m_root.GetRootList(t,tr,w);
					// current_rootlist is the number of RootOverTime for a window

					// write global offset: <num-sessions> gx0 gy0 gx1 gy1			
					fprintf(file,"%d",nsessions);
					for( g = 0; g < nsessions; g++) 
               {
//				      for( i = 0; i < current_rootlist->size(); i++ ) 
//							offset = current_rootlist->at(i).GetOffset(g);
//							//if( current_rootlist->at(i).GetOffset(g).x > 0 ) 
//							//	offset = current_rootlist->at(i).GetOffset(g);
						offset = current_rootlist->offset_list.at(g);
						fprintf(file," %d %d", offset.x, offset.y);
					}
					fprintf(file,"\n");
					
					// write root block size
					num_rootblocks = current_rootlist->size(); // number of root over time
					if( num_rootblocks > 0 ) 
               {
						// write number of root blocks
						fprintf(file,"%d\n",num_rootblocks);
						// loop through root blocks
						for( rb = 0; rb < num_rootblocks; rb++ ) 
                  {
							status = current_rootlist->at(rb).GetStatus();
							if( status == true )
								temp_str.Format("live");
							else
								temp_str.Format("dead");
							// write root block
							fprintf(file,"%d %d %s\n",current_rootlist->at(rb).GetFirstSession(), current_rootlist->at(rb).GetNumInstances(),temp_str);
							for(s = current_rootlist->at(rb).GetFirstSession() - 1; s < current_rootlist->at(rb).GetNumInstances()+current_rootlist->at(rb).GetFirstSession()-1; s++)
							{
							   current_rootinstance = current_rootlist->at(rb).GetRootInstance(s,&status);
							   num_points = current_rootinstance->points.size();
							   // write number of points
							   fprintf(file,"%d",num_points);
							   // loop through points
							   for( p = 0; p < num_points; p++ ) 
                        {
							      fprintf(file," %d %d",current_rootinstance->points.at(p).x,current_rootinstance->points.at(p).y);
							   } // end p
							   // write <diameter(xc,yc)>
							   fprintf(file," %d %d %.2f ",current_rootinstance->diameter_center.x,current_rootinstance->diameter_center.y,current_rootinstance->diameter);
							   // write color
							   root_color = current_rootinstance->color;
							   if( root_color == COLOR_DARKBROWN )
                           temp_str = "darkbrown";
                        else if( root_color == COLOR_LIGHTBROWN )
                           temp_str = "lightbrown";
                        else if( root_color == COLOR_WHITE )
                           temp_str = "white";
                        else if( root_color == COLOR_BLACK )
                           temp_str = "black";
 
							  fprintf(file,"%s\n", temp_str);
                     } // end s
						} // end r
					}
					else 
               {
					   // no data
						fprintf(file,"0\n");
					} // end else
				} // end w
			} // end tr
		} // end t
	
	   fclose(file);
	} // end if
	else 
   {
		AfxMessageBox("Write error");
	} // end else
}


void RootflyData::Export(CString& filename)
{
   FILE *file; 
	int ntubes,ntransects,nwindows,nsessions;
	int t,w,tr,rb,s;
	CString temp_str;
	RootList *current_rootlist;
	int num_rootblocks;
	RootInstance *current_rootinstance;
	CPoint offset;
	Color root_color;
	bool status;

   //make sure the filename entered is valid
   if( (file=fopen(filename,"w")) != NULL ) 
   {	
		// get the dimensions of the data structure
		m_root.GetDimensions(&ntubes,&ntransects,&nwindows,&nsessions);

      //write out headers
      fprintf(file," Tube, Transect, Window, Root number, Session appeared, Session died, Life span (in Sessions),");
      for(int i=1;i<nsessions+1;i++)
      {
         fprintf(file,"Length date(%d), Diameter date(%d), Color date(%d),",i,i,i); 
      }
      fprintf(file,"\n");
		
		// loop through tubes
		for( t = 0; t < ntubes; t++ ) 
      {
			// loop through transects
			for( tr = 0; tr < ntransects; tr++ ) 
         {
				// loop through windows
				for( w = 0; w < nwindows; w++ ) 
            {
					current_rootlist = m_root.GetRootList(t,tr,w);
					// current_rootlist is the number of RootOverTime for a window

					// if there are any rootblocks, deal with them
					num_rootblocks = current_rootlist->size(); // number of root over time
					if( num_rootblocks > 0 ) 
               {    

						// loop through each RootOverTime
						for( rb = 0; rb < num_rootblocks; rb++ ) 
                  {
							status = current_rootlist->at(rb).GetStatus();
							if( status == true )
								temp_str.Format("live");
							else
								temp_str.Format("dead");

                     // write tube and transect ( 01 A )
  			         temp_str.Format("%02d,%c",t+1,65+tr);
				         fprintf(file,"%s,",temp_str);

                     //write out window number
                     temp_str.Format("%02d",w+1);
                     fprintf(file,"%s,",temp_str);

                     //write out root number
                     temp_str.Format("%d",rb+1);
                     fprintf(file,"%s,",temp_str);

                     bool been_through_loop_once = false;
                     int instances = current_rootlist->at(rb).GetNumInstances();
                     int death_session;
							// write out every root in window
							//fprintf(file,"%d %d %s\n",current_rootlist->at(rb).GetFirstSession(), current_rootlist->at(rb).GetNumInstances(),temp_str);
							for(s = current_rootlist->at(rb).GetFirstSession() - 1; s < current_rootlist->at(rb).GetNumInstances()+current_rootlist->at(rb).GetFirstSession()-1; s++)
							{
                        //gets the exact root
							   current_rootinstance = current_rootlist->at(rb).GetRootInstance(s,&status);
						
                        //write out the first session that the root appeared and
                        //write out the when the root died and
                        //write out the roots lifespan
                        if( been_through_loop_once == false )
                        {
                           been_through_loop_once = true;

                           //writes out first session appeared
                           temp_str.Format("%d",s+1);
                           fprintf(file,"%s,",temp_str);

                           //writes out death date if the root has died
                           if( current_rootlist->at(rb).GetStatus() == false )
                           {
                              death_session = s + instances;
                              temp_str.Format("%d",death_session);
                              fprintf(file,"%s,",temp_str);
                           }
                           //root is still alive
                           else
                           {
                              temp_str = "Not Dead";
                              fprintf(file,"%s,",temp_str);
                           }

                           //writes out lifespan
                           temp_str.Format("%d",instances);
                           fprintf(file,"%s,",temp_str);

                           //put in blanks till root arrived
                           for(i=0;i< s;i++)
                           fprintf(file,".,.,.,");

                        }//end code that's run once


                        //write out length
                        float length = 0;
                        for(int k = 1; k < current_rootinstance->points.size(); k++)
                        {
                           length += sqrt(1.0*(current_rootinstance->points.at(k-1).x-current_rootinstance->points.at(k).x)*(current_rootinstance->points.at(k-1).x-current_rootinstance->points.at(k).x)+1.0*(current_rootinstance->points.at(k-1).y-current_rootinstance->points.at(k).y)*(current_rootinstance->points.at(k-1).y-current_rootinstance->points.at(k).y));
                        } //end for
                        temp_str.Format("%.2f",length/m_mm_conversion);
                        fprintf(file,"%s,",temp_str);

                        //write out diameter
							   fprintf(file,"%.2f, ",current_rootinstance->diameter/m_mm_conversion);

							   // write color
							   root_color = current_rootinstance->color;
							   if( root_color == COLOR_DARKBROWN )
                           temp_str = "darkbrown";
                        else if( root_color == COLOR_LIGHTBROWN )
                           temp_str = "lightbrown";
                        else if( root_color == COLOR_WHITE )
                           temp_str = "white";
                        else if( root_color == COLOR_BLACK )
                           temp_str = "black";
							   fprintf(file,"%s,", temp_str);
                     } // end s
                     fprintf(file,"\n");
						} // end rb
               } // if there are rootblocks
					else 
               {
					   // no data
						fprintf(file,"0\n");
					} // end else
				} // end w
			} // end tr
		} // end t
	
	   fclose(file);
	} // end if
	else 
   {
		AfxMessageBox("Write error");
	} // end else
}
// -------------------------------------------------------------------

//modified flag used to keep track of changes being made to the structure
//sets the modified flag
void RootflyData::SetModify(bool t)
{
   m_modified = t;
}
//returns the modified flag
bool RootflyData::GetModify() const
{
   return m_modified;
}