#include "Session_Array.h"

#define TABLE_SIZE 10

#include <iostream>
#include <vector>
#include <algorithm>
#include <assert.h>

using std::cout;

// ----------------------------------------------------------------------------------------------------------

// returns one-based index of the day since the year began
// (Jan 1 is day 1, Jan 2 is day 2, etc.)
// Takes leap years into account.
int Date::GetDayOfYear() const
{
  assert(month >= 1 && month <= 12);
  assert(day >= 1 && day <= 31);
  assert(year >= 1970);
  int month_length[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 30, 31, 31};
  if ((year % 4 == 0) && ((year % 100 != 0) || (year % 400 == 0)))  
    { month_length[1]++;  /* leap year */ }
  int d = day;
  for (int i = 0 ; i < month-1 ; i++)  d += month_length[i];
  return d;
}

CString Date::GetMonth() const
{
  assert(month >= 1 && month <= 12);
  switch (month)
  {
  case  1:  return "Jan.";
  case  2:  return "Feb.";
  case  3:  return "March";
  case  4:  return "April";
  case  5:  return "May";
  case  6:  return "June";
  case  7:  return "July";
  case  8:  return "Aug.";
  case  9:  return "Sept.";
  case 10:  return "Oct.";
  case 11:  return "Nov.";
  case 12:  return "Dec.";
//  case  1:  return "January";
//  case  2:  return "February";
//  case  3:  return "March";
//  case  4:  return "April";
//  case  5:  return "May";
//  case  6:  return "June";
//  case  7:  return "July";
//  case  8:  return "August";
//  case  9:  return "September";
//  case 10:  return "October";
//  case 11:  return "November";
//  case 12:  return "December";
  default:  return "Error";
  }
}

// return the number of days in a specific year
int GetNumberOfDaysInYear(int year)
{
  assert(year >= 1970);
  Date date( year, 12, 31 );  // December 31 (last day of the year)
  return date.GetDayOfYear();
}

// returns one-based index of the day since Jan 1, 1970.
int Date::GetDaySince1970() const
{
  int day = 0;
  {
    const int first_year = 1970;
    for (int y = first_year ; y < year ; y++)
    {
      day += GetNumberOfDaysInYear( y );
    }
  }
  day += GetDayOfYear();
  return day;
}

// ----------------------------------------------------------------------------------------------------------

Session_Array::Session_Array(){ } // constructor

// Inputs an int date.  If the date is in the vector, return where, else
//    insert the date, resort the vector, return where date is.
int Session_Array::add_date(const Date& date){

   int result;

   result = CheckForDate(date);

   if(result != -1)
	   return result;

   dates.push_back(date);

   SortArray();

   result = CheckForDate(date);

   return result;

} // add_date()

// ----------------------------------------------------------------------------------------------------------

// Returns the date in the vector at the position specified
// Returns default date (year = month = day = -1) if date not found
Date Session_Array::GetDate(int sessionNumber)
{
  if (sessionNumber <= 0 || sessionNumber > dates.size())  return Date();
  else return dates.at(sessionNumber-1);
} // GetDate()

// ----------------------------------------------------------------------------------------------------------

// Checks vector for date. Returns -1 if date is not found, otherwise, returns
//    current location of date in vector.
int Session_Array::CheckForDate(const Date& date)
{
  const int tolerance = 2;  // any dates within this number of days is considered a single session
  int day1 = date.GetDaySince1970();
  int day2;
	for(int x=0 ; x < dates.size() ; x++)
  {
    day2 = dates[x].GetDaySince1970();
    if(day1 >= day2 && day1 <= day2 + tolerance)
      return x;
  }
	return -1;
} // CheckForDate()

//int Session_Array::CheckForDate(const Date& date){
//  int input_dayOfYear = date.GetDayOfYear();
//  int curr_dayOfYear;
//	for(int x=0; x < dates.size(); x++){
//    curr_dayOfYear = dates[x].GetDayOfYear();
//    if(input_dayOfYear >= curr_dayOfYear && input_dayOfYear <= curr_dayOfYear + 2)
//      return x;
//  }
//	return -1;
//} // CheckForDate()

// ----------------------------------------------------------------------------------------------------------

void Session_Array::SortArray(){
  std::sort(dates.begin(), dates.end());
} // SortArray()

//int Session_Array::GetDayOfYear(const Date& date)
//{
//  assert(date.month >= 1 && month <= 12);
//  assert(date.day >= 1 && day <= 31);
//  int month_length[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 30, 31, 31};
//  if ((year % 4 == 0) && ((year % 100 != 0) || (year % 400 == 0)))  
//  { month_length[1]++;  /* leap year */ }
//  for (int i = 0 ; i < month-1 ; i++)  day += month_length[i];
//  return day;
//}
//
//// return day since Christ's birth
//int GetDayAnnoDomini(const Date& date)
//{
//  int day_of_year = GetDayOfYear(
//}